import {Language, LanguageStatusMessageMap} from '../../../common';
import {MiddlewareResponseError, Response} from '../../common';

interface PlainResponseParams<T = unknown> extends Response {
	body?: T;
}

interface HttpMiddlewareErrorParams<T = unknown> extends Omit<PlainResponseParams<T>, 'statusMessage'> {
	cause?: unknown
}

export class HttpMiddlewareError<T = unknown> extends MiddlewareResponseError implements PlainResponseParams<T> {
	body?: T;

	constructor(statusMessage: keyof Language['statusMessages'], params: HttpMiddlewareErrorParams<T>) {
		super(statusMessage, params);
		this.body = params.body;
	}
}

export class PlainResponse<T = unknown> implements Response {
	readonly statusCode: Response['statusCode'];

	readonly statusMessage?: keyof LanguageStatusMessageMap;

	readonly headers: Response['headers'];

	readonly body?: T;

	constructor(args: PlainResponseParams<T>) {
		this.statusCode = args.statusCode;
		this.statusMessage = args.statusMessage;
		this.headers = args.headers;
		this.body = args.body;
	}
}
